% Polynomial regression (NROSA-OHA)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%  Noise Reduced Reaction-Oriented Sensitivity Analysis  %%%%%%%%%
%%%%%%%%%%                 OHA COMPUTATION                        %%%%%%%%%
%%%%%%%%%%             Authors: Zhang/Goutsias                    %%%%%%%%%
%%%%%%%%%%            Last Modified: 09/17/2010                   %%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

Q      = M + P;
lambda = repmat(0,[1,Q]);
lambda(1:M) = rlambda;
lambda((M+1):Q) = sigma;

alphaTT1 = repmat(0,[Q,Q]);
alphaDD1 = repmat(0,[Q,Q]);
alphaSS1 = repmat(0,[Q,Q]);
alphaRR1 = repmat(0,[Q,Q]);
alphaTT2 = repmat(0,[Q,Q]);
alphaDD2 = repmat(0,[Q,Q]);
alphaSS2 = repmat(0,[Q,Q]);
alphaRR2 = repmat(0,[Q,Q]);
alphaTT3 = repmat(0,[Q,Q]);
alphaDD3 = repmat(0,[Q,Q]);
alphaSS3 = repmat(0,[Q,Q]);
alphaRR3 = repmat(0,[Q,Q]);
alphaTT4 = repmat(0,[Q,Q]);
alphaDD4 = repmat(0,[Q,Q]);
alphaSS4 = repmat(0,[Q,Q]);
alphaRR4 = repmat(0,[Q,Q]);

perm = repmat(0,[Q,L]);

TT = repmat(0,[L,1]);
DD = repmat(0,[L,1]);
SS = repmat(0,[L,1]);
RR = repmat(0,[L,1]);

S = 2*Q^2+1;  
W = repmat(1,[L,S]);

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% MC-LHS iterations
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Determine LHS permutations
%

for q = 1:Q
    perm(q,:) = randperm(L);
end

for iter = 1:L
    
    iterleft = L+1-iter;
    fprintf('Remaining MC-LHS iterations: %6.0f\n',iterleft)
    
    %
    % Calculate Latin hypercube samples
    %
    
    uniform = rand(Q,1);
    index   = (perm(:,iter)-1+uniform)./L;
    YR      = repmat(0,[1,M]);
    YR      = (rlambda*(-sqrt(2)*erfcinv(2*index(1:M))))';   % inverse normal CDF
    temp    = repmat(YR,[2 1]);   % replicate each element of YR twice
    YRR     = temp(:);            % replicate each element of YR twice
    YRR(8)  = 0;
    YRR(16) = 0;
    YRR(32) = 0;
    Z       = repmat(0,[P,1]);
    Z       = sigma*(-sqrt(2)*erfcinv(2*index((M+1):Q))); % inverse normal CDF
    svyn    = V*Z;
    kappan  = kappa.*exp(-YRR).*exp(svyn);
    YRZ     = [YR, Z'];
    
    %
    % Calculate response characteristics (duration, integrated response, and strength)
    %
    
    if iode == 1
        try
            [t,x] = egfr(kappan,x0,[odetol,odetol],timings);
        catch
            [t,x] = ode15s(@netode_EGFR,tpoints,x0,options,kappan,stoich);
        end
    else
        [t,x] = ode15s(@netode_EGFR,tpoints,x0,options,kappan,stoich);
    end
    
    [TT(iter),DD(iter),SS(iter),RR(iter)] = featureit_EGFR(x,t,tresol,durtol,odetol);
    
    %
    % Calculate polynomical regression matrix
    %
    
    W(iter,(2:(Q+1)))       = YRZ./lambda;
    W(iter,((Q+2):(2*Q+1))) = ((YRZ./lambda).^2 - 1)./sqrt(2);
    colm = 2*Q + 1;
    
    for q = 1:(Q-1)
        for qq = (q+1):Q
            colm = colm + 1;
            W(iter,colm) = YRZ(q)*YRZ(qq)/(lambda(q)*lambda(qq));
        end
    end
    
    for q = 1:(Q-1)
        for qq = (q+1):Q
            colm = colm + 1;
            W(iter,colm) = (((YRZ(q)/lambda(q))^2 - 1)/sqrt(2))*(YRZ(qq)/lambda(qq));
        end
    end
    
    for q = 1:(Q-1)
        for qq = (q+1):Q
            colm = colm + 1;
            W(iter,colm) = (YRZ(q)/lambda(q))*(((YRZ(qq)/lambda(qq))^2 - 1)/sqrt(2));
        end
    end
    
    for q = 1:(Q-1)
        for qq = (q+1):Q
            colm = colm + 1;
            W(iter,colm) = ((YRZ(q)/lambda(q))^2 - 1)*((YRZ(qq)/lambda(qq))^2 - 1)/2;
        end
    end
    
end

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Estimate polynomial coefficients
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

alpha_TT = inv(W'*W)*(W'*TT);
alpha_DD = inv(W'*W)*(W'*DD);
alpha_SS = inv(W'*W)*(W'*SS);
alpha_RR = inv(W'*W)*(W'*RR);

%
% First-order coefficients
%

alphaT1 = alpha_TT(2:(Q+1));    
alphaD1 = alpha_DD(2:(Q+1));
alphaS1 = alpha_SS(2:(Q+1));
alphaR1 = alpha_RR(2:(Q+1));

alphaT2 = alpha_TT((Q+2):(2*Q+1));
alphaD2 = alpha_DD((Q+2):(2*Q+1));
alphaS2 = alpha_SS((Q+2):(2*Q+1));
alphaR2 = alpha_RR((Q+2):(2*Q+1));

%
% Second-order coefficients
%

vecplc = zeros(Q,Q);
row    = 2*Q + 1;

for q = 1:(Q-1)
    for qq = (q+1):Q
        row = row + 1;
        vecplc(q,qq) = row;
    end
end

for q = 1:(Q-1)
    for qq = (q+1):Q
        alphaTT1(q,qq) = alpha_TT(vecplc(q,qq));
        alphaDD1(q,qq) = alpha_DD(vecplc(q,qq));
        alphaSS1(q,qq) = alpha_SS(vecplc(q,qq));
        alphaRR1(q,qq) = alpha_RR(vecplc(q,qq));
    end
end

for q = 1:(Q-1)
    for qq = (q+1):Q
        row = row + 1;
        vecplc(q,qq) = row;
    end
end

for q = 1:(Q-1)
    for qq = (q+1):Q
        alphaTT2(q,qq) = alpha_TT(vecplc(q,qq));
        alphaDD2(q,qq) = alpha_DD(vecplc(q,qq));
        alphaSS2(q,qq) = alpha_SS(vecplc(q,qq));
        alphaRR2(q,qq) = alpha_RR(vecplc(q,qq));
    end
end

for q = 1:(Q-1)
    for qq = (q+1):Q
        row = row + 1;
        vecplc(q,qq) = row;
    end
end

for q = 1:(Q-1)
    for qq = (q+1):Q
        alphaTT3(q,qq) = alpha_TT(vecplc(q,qq));
        alphaDD3(q,qq) = alpha_DD(vecplc(q,qq));
        alphaSS3(q,qq) = alpha_SS(vecplc(q,qq));
        alphaRR3(q,qq) = alpha_RR(vecplc(q,qq));
    end
end

for q = 1:(Q-1)
    for qq = (q+1):Q
        row = row + 1;
        vecplc(q,qq) = row;
    end
end

for q = 1:(Q-1)
    for qq = (q+1):Q
        alphaTT4(q,qq) = alpha_TT(vecplc(q,qq));
        alphaDD4(q,qq) = alpha_DD(vecplc(q,qq));
        alphaSS4(q,qq) = alpha_SS(vecplc(q,qq));
        alphaRR4(q,qq) = alpha_RR(vecplc(q,qq));
    end
end